#ifndef __NET_CFG80211_H
#define __NET_CFG80211_H

#include <linux/netlink.h>
#include <linux/skbuff.h>
#include <net/genetlink.h>

/*
 * 802.11 configuration in-kernel interface
 *
 * Copyright 2006 Johannes Berg <johannes@sipsolutions.net>
 */

/**
 * struct scan_channel - describes a single channel to scan
 * @phymode: PHY mode for this channel
 * @channel: channel number (1-14, ...)
 * @active: scan actively or passively on this channel
 */
struct scan_channel {
	int phymode;
	u32 channel;
	int active;
};

/**
 * struct scan_params - describes scan parameters
 * @n_channels: number of items in @channels array or -1 to indicate all
 * 	channels should be scanned (in that case @channels will be %NULL)
 * @active: when n_channels is -1 this determines active/passive scanning.
 * @phymode: when n_channels is -1 this determines PHY mode to scan. It is
 *	not possible to scan different PHY modes in one request w/o giving
 *	a channel list.
 * @channels: array containing @n_channels &struct scan_channel items
 */
struct scan_params {
	int n_channels;
	int active;
	int phymode;
	struct scan_channel *channels;
};

/**
 * struct association_params - describes association parameters
 * @valid: this member contains flags which items are valid
 * @bssid: the BSSID of the BSS to associate [%ASSOC_PARAMS_BSSID]
 * @timeout: timeout (in TU) [%ASSOC_PARAMS_TIMEOUT]
 * @ie: information element(s) to include in the association frames [%ASSOC_PARAMS_IE]
 * @ie_len: length of the information element(s)
 * @ssid: the SSID, always valid.
 * @ssid_len: length of the SSID
 */
struct association_params {
	u8 *bssid;
	u32 timeout;
	u8 *ie;
	int ie_len;
	u8 *ssid;
	int ssid_len;

	unsigned int valid;
};
#define ASSOC_PARAMS_TIMEOUT	(1<<0)

/**
 * struct key_params - key information
 */
struct key_params {
	u8 *key;
	int key_len;
	int key_id;
	u32 key_type;
	u8 *macaddress;
	u32 cipher;
};

/* from net/wireless.h */
struct wiphy;

/**
 * struct cfg80211_ops - backend description for wireless configuration
 *
 * This struct is registered by fullmac card drivers and/or wireless stacks
 * in order to handle configuration requests on their interfaces.
 *
 * All callbacks except where otherwise noted should return 0
 * on success or a negative error code.
 *
 * All operations are currently invoked under rtnl for consistency with the
 * wireless extensions but this is subject to reevaluation as soon as this
 * code is used more widely and we have a first user without wext.
 *
 * @add_virtual_intf: create a new virtual interface with the given name
 *
 * @del_virtual_intf: remove the virtual interface determined by ifindex.
 *
 * @change_virtual_intf: change type of virtual interface
 *
 * @associate: associate with given parameters
 *
 * @disassociate: disassociate from current AP
 *
 * @deauth: deauth from current AP
 *
 * @initiate_scan: scan with the given information (see &struct scan_params above)
 *
 * @get_association: get BSSID of the BSS that the device is currently
 *		     associated to and return 1, or return 0 if not
 *		     associated (or a negative error code)
 * @get_auth_list: get list of BSSIDs of all BSSs the device has
 *		   authenticated with, must call next_bssid for each,
 *		   next_bssid returns non-zero on error, the given data
 *		   is to be passed to that callback
 * @add_key: add a key using &struct key_params
 * @del_key: delete a key using info from &struct key_params
 */
struct cfg80211_ops {
	int	(*add_virtual_intf)(struct wiphy *wiphy, char *name,
				    unsigned int type);
	int	(*del_virtual_intf)(struct wiphy *wiphy, int ifindex);
	int	(*change_virtual_intf)(struct wiphy *wiphy, int ifindex,
				       unsigned int type);

	int	(*associate)(struct wiphy *wiphy, struct net_device *dev,
			     struct association_params *params);
	int	(*disassociate)(struct wiphy *wiphy, struct net_device *dev);
	int	(*deauth)(struct wiphy *wiphy, struct net_device *dev);


	int	(*initiate_scan)(struct wiphy *wiphy, struct net_device *dev,
				 struct scan_params *params);


	int	(*get_association)(struct wiphy *wiphy, struct net_device *dev,
				   u8 *bssid);

	int	(*get_auth_list)(struct wiphy *wiphy, struct net_device *dev,
				 void *data,
				 int (*next_bssid)(void *data, u8 *bssid));

	int	(*add_key)(struct wiphy *wiphy, struct net_device *dev,
			   struct key_params *params);
	int	(*del_key)(struct wiphy *wiphy, struct net_device *dev,
			   struct key_params *params);
};


/* helper functions specific to nl80211 */
extern void *nl80211hdr_put(struct sk_buff *skb, u32 pid,
			    u32 seq, int flags, u8 cmd);
extern void *nl80211msg_new(struct sk_buff **skb, u32 pid,
			    u32 seq, int flags, u8 cmd);

#endif /* __NET_CFG80211_H */
